@@ -83,12 +83,14 @@ static void pango_xft_renderer_end            (PangoRenderer    *renderer);
 static void flush_trapezoids (PangoXftRenderer *xftrenderer);
 static void flush_glyphs (PangoXftRenderer *xftrenderer);
 
-G_DEFINE_TYPE_WITH_PRIVATE (PangoXftRenderer, pango_xft_renderer, PANGO_TYPE_RENDERER)
+G_DEFINE_TYPE (PangoXftRenderer, pango_xft_renderer, PANGO_TYPE_RENDERER)
 
 static void
 pango_xft_renderer_init (PangoXftRenderer *xftrenderer)
 {
-  xftrenderer->priv = pango_xft_renderer_get_instance_private (xftrenderer);
+  xftrenderer->priv = G_TYPE_INSTANCE_GET_PRIVATE (xftrenderer,
+						   PANGO_TYPE_XFT_RENDERER,
+						   PangoXftRendererPrivate);
   xftrenderer->priv->alpha = 0xffff;
 }
 
@@ -120,6 +122,8 @@ pango_xft_renderer_class_init (PangoXftRendererClass *klass)
 						     "The screen being rendered to",
 						     0, G_MAXINT, 0,
 						     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+
+  g_type_class_add_private (object_class, sizeof (PangoXftRendererPrivate));
 }
 
 static void
@@ -185,7 +189,7 @@ flush_glyphs (PangoXftRenderer *xftrenderer)
 
   PANGO_XFT_RENDERER_GET_CLASS (xftrenderer)->composite_glyphs (xftrenderer,
 								xft_font,
-								(XftGlyphSpec *)(void*)xftrenderer->priv->glyphs->data,
+								(XftGlyphSpec *)xftrenderer->priv->glyphs->data,
 								xftrenderer->priv->glyphs->len);
 
   g_array_set_size (xftrenderer->priv->glyphs, 0);
@@ -527,7 +531,7 @@ flush_trapezoids (PangoXftRenderer *xftrenderer)
 
   PANGO_XFT_RENDERER_GET_CLASS (xftrenderer)->composite_trapezoids (xftrenderer,
 								    xftrenderer->priv->trapezoid_part,
-								    (XTrapezoid *)(void *)xftrenderer->priv->trapezoids->data,
+								    (XTrapezoid *)xftrenderer->priv->trapezoids->data,
 								    xftrenderer->priv->trapezoids->len);
 
   g_array_set_size (xftrenderer->priv->trapezoids, 0);
